# Generated by Django 4.2.4 on 2023-10-05 12:57

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("df_auth", "0002_alter_user2fa_user_userregistration"),
    ]

    operations = [
        migrations.AlterField(
            model_name="userregistration",
            name="is_registering",
            field=models.BooleanField(default=True),
        ),
    ]
